@@ -18,7 +18,7 @@ class AnnotationBuilder:
     The AnnotationBuilder creates dictionaries representing PDF annotations.
 
     Those dictionaries can be modified before they are added to a PdfWriter
-    instance via `writer.add_annotation`.
+    instance via ``writer.add_annotation``.
 
     See `adding PDF annotations <../user/adding-pdf-annotations.html>`_ for
     it's usage combined with PdfWriter.
@@ -37,14 +37,14 @@ class AnnotationBuilder:
         Add text annotation.
 
         Args:
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]``
-            specifying the clickable rectangular area
-          text: The text that is added to the document
-          open:
-          flags:
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]``
+                specifying the clickable rectangular area
+            text: The text that is added to the document
+            open:
+            flags:
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         # TABLE 8.23 Additional entries specific to a text annotation
         text_obj = DictionaryObject(
@@ -68,26 +68,28 @@ class AnnotationBuilder:
         italic: bool = False,
         font_size: str = "14pt",
         font_color: str = "000000",
-        border_color: str = "000000",
-        background_color: str = "ffffff",
+        border_color: Optional[str] = "000000",
+        background_color: Optional[str] = "ffffff",
     ) -> DictionaryObject:
         """
         Add text in a rectangle to a page.
 
         Args:
-          text: Text to be added
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]``
-            specifying the clickable rectangular area
-          font: Name of the Font, e.g. 'Helvetica'
-          bold: Print the text in bold
-          italic: Print the text in italic
-          font_size: How big the text will be, e.g. '14pt'
-          font_color: Hex-string for the color, e.g. cdcdcd
-          border_color: Hex-string for the border color, e.g. cdcdcd
-          background_color: Hex-string for the background of the annotation, e.g. cdcdcd
+            text: Text to be added
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]``
+                specifying the clickable rectangular area
+            font: Name of the Font, e.g. 'Helvetica'
+            bold: Print the text in bold
+            italic: Print the text in italic
+            font_size: How big the text will be, e.g. '14pt'
+            font_color: Hex-string for the color, e.g. cdcdcd
+            border_color: Hex-string for the border color, e.g. cdcdcd.
+                Use ``None`` for no border.
+            background_color: Hex-string for the background of the annotation,
+                e.g. cdcdcd. Use ``None`` for transparent background.
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         font_str = "font: "
         if bold is True:
@@ -97,10 +99,11 @@ class AnnotationBuilder:
         font_str = f"{font_str}{font} {font_size}"
         font_str = f"{font_str};text-align:left;color:#{font_color}"
 
-        bg_color_str = ""
-        for st in hex_to_rgb(border_color):
-            bg_color_str = f"{bg_color_str}{st} "
-        bg_color_str = f"{bg_color_str}rg"
+        default_appearance_string = ""
+        if border_color:
+            for st in hex_to_rgb(border_color):
+                default_appearance_string = f"{default_appearance_string}{st} "
+            default_appearance_string = f"{default_appearance_string}rg"
 
         free_text = DictionaryObject()
         free_text.update(
@@ -111,14 +114,21 @@ class AnnotationBuilder:
                 NameObject("/Contents"): TextStringObject(text),
                 # font size color
                 NameObject("/DS"): TextStringObject(font_str),
-                # border color
-                NameObject("/DA"): TextStringObject(bg_color_str),
-                # background color
-                NameObject("/C"): ArrayObject(
-                    [FloatObject(n) for n in hex_to_rgb(background_color)]
-                ),
+                NameObject("/DA"): TextStringObject(default_appearance_string),
             }
         )
+        if border_color is None:
+            # Border Style
+            free_text[NameObject("/BS")] = DictionaryObject(
+                {
+                    # width of 0 means no border
+                    NameObject("/W"): NumberObject(0)
+                }
+            )
+        if background_color is not None:
+            free_text[NameObject("/C")] = ArrayObject(
+                [FloatObject(n) for n in hex_to_rgb(background_color)]
+            )
         return free_text
 
     @staticmethod
@@ -133,16 +143,16 @@ class AnnotationBuilder:
         Draw a line on the PDF.
 
         Args:
-          p1: First point
-          p2: Second point
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]``
-            specifying the clickable rectangular area
-          text: Text to be displayed as the line annotation
-          title_bar: Text to be displayed in the title bar of the
-            annotation; by convention this is the name of the author
+            p1: First point
+            p2: Second point
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]``
+                specifying the clickable rectangular area
+            text: Text to be displayed as the line annotation
+            title_bar: Text to be displayed in the title bar of the
+                annotation; by convention this is the name of the author
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         line_obj = DictionaryObject(
             {
@@ -187,13 +197,13 @@ class AnnotationBuilder:
         This method uses the /Square annotation type of the PDF format.
 
         Args:
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]``
-            specifying the clickable rectangular area
-          interiour_color: None or hex-string for the color, e.g. cdcdcd
-            If None is used, the interiour is transparent.
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]``
+                specifying the clickable rectangular area
+            interiour_color: None or hex-string for the color, e.g. cdcdcd
+                If None is used, the interiour is transparent.
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         square_obj = DictionaryObject(
             {
@@ -221,13 +231,13 @@ class AnnotationBuilder:
         This method uses the /Circle annotation type of the PDF format.
 
         Args:
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]`` specifying
-            the bounding box of the ellipse
-          interiour_color: None or hex-string for the color, e.g. cdcdcd
-            If None is used, the interiour is transparent.
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]`` specifying
+                the bounding box of the ellipse
+            interiour_color: None or hex-string for the color, e.g. cdcdcd
+                If None is used, the interiour is transparent.
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         ellipse_obj = DictionaryObject(
             {
@@ -288,22 +298,22 @@ class AnnotationBuilder:
         An internal link requires the target_page_index, fit, and fit args.
 
         Args:
-          rect: array of four integers ``[xLL, yLL, xUR, yUR]``
-            specifying the clickable rectangular area
-          border: if provided, an array describing border-drawing
-            properties. See the PDF spec for details. No border will be
-            drawn if this argument is omitted.
-            - horizontal corner radius,
-            - vertical corner radius, and
-            - border width
-            - Optionally: Dash
-          url: Link to a website (if you want to make an external link)
-          target_page_index: index of the page to which the link should go
-            (if you want to make an internal link)
-          fit: Page fit or 'zoom' option.
+            rect: array of four integers ``[xLL, yLL, xUR, yUR]``
+                specifying the clickable rectangular area
+            border: if provided, an array describing border-drawing
+                properties. See the PDF spec for details. No border will be
+                drawn if this argument is omitted.
+                - horizontal corner radius,
+                - vertical corner radius, and
+                - border width
+                - Optionally: Dash
+            url: Link to a website (if you want to make an external link)
+            target_page_index: index of the page to which the link should go
+                (if you want to make an internal link)
+            fit: Page fit or 'zoom' option.
 
         Returns:
-          A dictionary object representing the annotation.
+            A dictionary object representing the annotation.
         """
         from ..types import BorderArrayType
 
