@@ -34,7 +34,7 @@ import re
 from io import BytesIO
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, cast
 
-from .._protocols import PdfWriterProtocol
+from .._protocols import PdfReaderProtocol, PdfWriterProtocol
 from .._utils import (
     WHITESPACES,
     StreamType,
@@ -82,7 +82,7 @@ class ArrayObject(list, PdfObject):
         force_duplicate: bool = False,
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "ArrayObject":
-        """clone object into pdf_dest"""
+        """Clone object into pdf_dest."""
         try:
             if self.indirect_reference.pdf == pdf_dest and not force_duplicate:  # type: ignore
                 return self
@@ -91,8 +91,6 @@ class ArrayObject(list, PdfObject):
         arr = cast("ArrayObject", self._reference_clone(ArrayObject(), pdf_dest))
         for data in self:
             if isinstance(data, StreamObject):
-                # if not hasattr(data, "indirect_reference"):
-                #    data.indirect_reference = None
                 dup = data._reference_clone(
                     data.clone(pdf_dest, force_duplicate, ignore_fields), pdf_dest
                 )
@@ -104,10 +102,7 @@ class ArrayObject(list, PdfObject):
         return cast("ArrayObject", arr)
 
     def items(self) -> Iterable[Any]:
-        """
-        Emulate DictionaryObject.items for a list
-        (index, object)
-        """
+        """Emulate DictionaryObject.items for a list (index, object)."""
         return enumerate(self)
 
     def write_to_stream(
@@ -128,9 +123,9 @@ class ArrayObject(list, PdfObject):
     @staticmethod
     def read_from_stream(
         stream: StreamType,
-        pdf: Any,
+        pdf: Optional[PdfReaderProtocol],
         forced_encoding: Union[None, str, List[str], Dict[int, str]] = None,
-    ) -> "ArrayObject":  # PdfReader
+    ) -> "ArrayObject":
         arr = ArrayObject()
         tmp = stream.read(1)
         if tmp != b"[":
@@ -152,7 +147,7 @@ class ArrayObject(list, PdfObject):
 
     @staticmethod
     def readFromStream(
-        stream: StreamType, pdf: Any  # PdfReader
+        stream: StreamType, pdf: PdfReaderProtocol
     ) -> "ArrayObject":  # deprecated
         deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
         return ArrayObject.read_from_stream(stream, pdf)
@@ -165,7 +160,7 @@ class DictionaryObject(dict, PdfObject):
         force_duplicate: bool = False,
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "DictionaryObject":
-        """clone object into pdf_dest"""
+        """Clone object into pdf_dest."""
         try:
             if self.indirect_reference.pdf == pdf_dest and not force_duplicate:  # type: ignore
                 return self
@@ -189,13 +184,13 @@ class DictionaryObject(dict, PdfObject):
         ignore_fields: Union[Tuple[str, ...], List[str]],
     ) -> None:
         """
-        Update the object from src
+        Update the object from src.
 
         Args:
-          src: "DictionaryObject":
-          pdf_dest:
-          force_duplicate:
-          ignore_fields:
+            src: "DictionaryObject":
+            pdf_dest:
+            force_duplicate:
+            ignore_fields:
         """
         #  First check if this is a chain list, we need to loop to prevent recur
         if (
@@ -234,7 +229,7 @@ class DictionaryObject(dict, PdfObject):
                                     cur_obj = cast("DictionaryObject", cur_obj[k])
                             except Exception:
                                 cur_obj = None
-                        for (s, c) in objs:
+                        for s, c in objs:
                             c._clone(s, pdf_dest, force_duplicate, ignore_fields + [k])
 
         for k, v in src.items():
@@ -276,8 +271,8 @@ class DictionaryObject(dict, PdfObject):
     @property
     def xmp_metadata(self) -> Optional[PdfObject]:
         """
-        Retrieve XMP (Extensible Metadata Platform) data relevant to the
-        this object, if available.
+        Retrieve XMP (Extensible Metadata Platform) data relevant to the this
+        object, if available.
 
         Stability: Added in v1.12, will exist for all future v1.x releases.
 
@@ -285,7 +280,6 @@ class DictionaryObject(dict, PdfObject):
           Returns a {@link #xmp.XmpInformation XmlInformation} instance
           that can be used to access XMP metadata from the document.  Can also
           return None if no metadata was found on the document root.
-
         """
         from ..xmp import XmpInformation
 
@@ -303,9 +297,9 @@ class DictionaryObject(dict, PdfObject):
         self,
     ) -> Optional[PdfObject]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :meth:`xmp_metadata` instead.
 
-            Use :meth:`xmp_metadata` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("getXmpMetadata", "xmp_metadata", "3.0.0")
         return self.xmp_metadata
@@ -313,9 +307,9 @@ class DictionaryObject(dict, PdfObject):
     @property
     def xmpMetadata(self) -> Optional[PdfObject]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :meth:`xmp_metadata` instead.
 
-            Use :meth:`xmp_metadata` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("xmpMetadata", "xmp_metadata", "3.0.0")
         return self.xmp_metadata
@@ -340,12 +334,12 @@ class DictionaryObject(dict, PdfObject):
     @staticmethod
     def read_from_stream(
         stream: StreamType,
-        pdf: Any,  # PdfReader
+        pdf: Optional[PdfReaderProtocol],
         forced_encoding: Union[None, str, List[str], Dict[int, str]] = None,
     ) -> "DictionaryObject":
         def get_next_obj_pos(
-            p: int, p1: int, rem_gens: List[int], pdf: Any
-        ) -> int:  # PdfReader
+            p: int, p1: int, rem_gens: List[int], pdf: PdfReaderProtocol
+        ) -> int:
             loc = pdf.xref[rem_gens[0]]
             for o in loc:
                 if p1 > loc[o] and p < loc[o]:
@@ -355,7 +349,9 @@ class DictionaryObject(dict, PdfObject):
             else:
                 return get_next_obj_pos(p, p1, rem_gens[1:], pdf)
 
-        def read_unsized_from_steam(stream: StreamType, pdf: Any) -> bytes:  # PdfReader
+        def read_unsized_from_steam(
+            stream: StreamType, pdf: PdfReaderProtocol
+        ) -> bytes:
             # we are just pointing at beginning of the stream
             eon = get_next_obj_pos(stream.tell(), 2**32, list(pdf.xref), pdf) - 1
             curr = stream.tell()
@@ -435,6 +431,7 @@ class DictionaryObject(dict, PdfObject):
             length = data[SA.LENGTH]
             if isinstance(length, IndirectObject):
                 t = stream.tell()
+                assert pdf is not None  # hint for mypy
                 length = pdf.get_object(length)
                 stream.seek(t, 0)
             pstart = stream.tell()
@@ -454,7 +451,7 @@ class DictionaryObject(dict, PdfObject):
                 if end == b"endstream":
                     # we found it by looking back one character further.
                     data["__streamdata__"] = data["__streamdata__"][:-1]
-                elif not pdf.strict:
+                elif pdf is not None and not pdf.strict:
                     stream.seek(pstart, 0)
                     data["__streamdata__"] = read_unsized_from_steam(stream, pdf)
                     pos = stream.tell()
@@ -475,7 +472,7 @@ class DictionaryObject(dict, PdfObject):
 
     @staticmethod
     def readFromStream(
-        stream: StreamType, pdf: Any  # PdfReader
+        stream: StreamType, pdf: PdfReaderProtocol
     ) -> "DictionaryObject":  # deprecated
         deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
         return DictionaryObject.read_from_stream(stream, pdf)
@@ -532,7 +529,6 @@ class TreeObject(DictionaryObject):
 
         child_obj = child.get_object()
         child = child.indirect_reference  # get_reference(child_obj)
-        # assert isinstance(child, IndirectObject)
 
         prev: Optional[DictionaryObject]
         if "/First" not in self:  # no child yet
@@ -583,10 +579,10 @@ class TreeObject(DictionaryObject):
         Adjust the pointers of the linked list and tree node count.
 
         Args:
-          prev:
-          prev_ref:
-          cur:
-          last:
+            prev:
+            prev_ref:
+            cur:
+            last:
         """
         next_ref = cur.get(NameObject("/Next"), None)
         if prev is None:
@@ -657,9 +653,7 @@ class TreeObject(DictionaryObject):
         _reset_node_tree_relationship(child_obj)
 
     def remove_from_tree(self) -> None:
-        """
-        remove the object from the tree it is in
-        """
+        """Remove the object from the tree it is in."""
         if NameObject("/Parent") not in self:
             raise ValueError("Removed child does not appear to be a tree item")
         else:
@@ -689,7 +683,7 @@ def _reset_node_tree_relationship(child_obj: Any) -> None:
     This resets the nodes attributes in respect to that tree.
 
     Args:
-      child_obj:
+        child_obj:
     """
     del child_obj[NameObject("/Parent")]
     if NameObject("/Next") in child_obj:
@@ -714,10 +708,10 @@ class StreamObject(DictionaryObject):
         Update the object from src.
 
         Args:
-          src:
-          pdf_dest:
-          force_duplicate:
-          ignore_fields:
+            src:
+            pdf_dest:
+            force_duplicate:
+            ignore_fields:
         """
         self._data = cast("StreamObject", src)._data
         try:
@@ -883,8 +877,8 @@ class ContentStream(DecodedStreamObject):
         self.pdf = pdf
 
         # The inner list has two elements:
-        #  [0] : List
-        #  [1] : str
+        #  Element 0: List
+        #  Element 1: str
         self.operations: List[Tuple[Any, Any]] = []
 
         # stream may be a StreamObject or an ArrayObject containing
@@ -916,12 +910,12 @@ class ContentStream(DecodedStreamObject):
         Clone object into pdf_dest.
 
         Args:
-          pdf_dest:
-          force_duplicate:
-          ignore_fields:
+            pdf_dest:
+            force_duplicate:
+            ignore_fields:
 
         Returns:
-          The cloned ContentStream
+            The cloned ContentStream
         """
         try:
             if self.indirect_reference.pdf == pdf_dest and not force_duplicate:  # type: ignore
@@ -948,16 +942,16 @@ class ContentStream(DecodedStreamObject):
         Update the object from src.
 
         Args:
-          src:
-          pdf_dest:
-          force_duplicate:
-          ignore_fields:
+            src:
+            pdf_dest:
+            force_duplicate:
+            ignore_fields:
         """
         self.pdf = pdf_dest
         self.operations = list(cast("ContentStream", src).operations)
         self.forced_encoding = cast("ContentStream", src).forced_encoding
-        # no need to call DictionaryObjection or any
-        # super(DictionaryObject,self)._clone(src, pdf_dest, force_duplicate, ignore_fields)
+        # no need to call DictionaryObjection or anything
+        # like super(DictionaryObject,self)._clone(src, pdf_dest, force_duplicate, ignore_fields)
         return
 
     def __parse_content_stream(self, stream: StreamType) -> None:
@@ -1095,7 +1089,7 @@ class ContentStream(DecodedStreamObject):
 
 def read_object(
     stream: StreamType,
-    pdf: Any,  # PdfReader
+    pdf: Optional[PdfReaderProtocol],
     forced_encoding: Union[None, str, List[str], Dict[int, str]] = None,
 ) -> Union[PdfObject, int, str, ContentStream]:
     tok = stream.read(1)
@@ -1106,7 +1100,6 @@ def read_object(
         # hexadecimal string OR dictionary
         peek = stream.read(2)
         stream.seek(-2, 1)  # reset to start
-
         if peek == b"<<":
             return DictionaryObject.read_from_stream(stream, pdf, forced_encoding)
         else:
@@ -1138,6 +1131,7 @@ def read_object(
         peek = stream.read(20)
         stream.seek(-len(peek), 1)  # reset to start
         if IndirectPattern.match(peek) is not None:
+            assert pdf is not None  # hint for mypy
             return IndirectObject.read_from_stream(stream, pdf)
         else:
             return NumberObject.read_from_stream(stream)
@@ -1168,6 +1162,13 @@ class Field(TreeObject):
                 self[NameObject(attr)] = data[attr]
             except KeyError:
                 pass
+        if isinstance(self.get("/V"), EncodedStreamObject):
+            d = cast(EncodedStreamObject, self[NameObject("/V")]).get_data()
+            if isinstance(d, bytes):
+                d = d.decode()
+            elif d is None:
+                d = ""
+            self[NameObject("/V")] = TextStringObject(d)
 
     # TABLE 8.69 Entries common to all field dictionaries
     @property
@@ -1178,9 +1179,9 @@ class Field(TreeObject):
     @property
     def fieldType(self) -> Optional[NameObject]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`field_type` instead.
 
-            Use :py:attr:`field_type` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("fieldType", "field_type", "3.0.0")
         return self.field_type
@@ -1208,9 +1209,9 @@ class Field(TreeObject):
     @property
     def altName(self) -> Optional[str]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`alternate_name` instead.
 
-            Use :py:attr:`alternate_name` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("altName", "alternate_name", "3.0.0")
         return self.alternate_name
@@ -1218,8 +1219,9 @@ class Field(TreeObject):
     @property
     def mapping_name(self) -> Optional[str]:
         """
-        Read-only property accessing the mapping name of this field. This
-        name is used by pypdf as a key in the dictionary returned by
+        Read-only property accessing the mapping name of this field.
+
+        This name is used by pypdf as a key in the dictionary returned by
         :meth:`get_fields()<pypdf.PdfReader.get_fields>`
         """
         return self.get(FieldDictionaryAttributes.TM)
@@ -1227,9 +1229,9 @@ class Field(TreeObject):
     @property
     def mappingName(self) -> Optional[str]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`mapping_name` instead.
 
-            Use :py:attr:`mapping_name` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("mappingName", "mapping_name", "3.0.0")
         return self.mapping_name
@@ -1245,8 +1247,9 @@ class Field(TreeObject):
     @property
     def value(self) -> Optional[Any]:
         """
-        Read-only property accessing the value of this field. Format
-        varies based on field type.
+        Read-only property accessing the value of this field.
+
+        Format varies based on field type.
         """
         return self.get(FieldDictionaryAttributes.V)
 
@@ -1258,9 +1261,9 @@ class Field(TreeObject):
     @property
     def defaultValue(self) -> Optional[Any]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`default_value` instead.
 
-            Use :py:attr:`default_value` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("defaultValue", "default_value", "3.0.0")
         return self.default_value
@@ -1269,17 +1272,18 @@ class Field(TreeObject):
     def additional_actions(self) -> Optional[DictionaryObject]:
         """
         Read-only property accessing the additional actions dictionary.
-        This dictionary defines the field's behavior in response to trigger events.
-        See Section 8.5.2 of the PDF 1.7 reference.
+
+        This dictionary defines the field's behavior in response to trigger
+        events. See Section 8.5.2 of the PDF 1.7 reference.
         """
         return self.get(FieldDictionaryAttributes.AA)
 
     @property
     def additionalActions(self) -> Optional[DictionaryObject]:  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`additional_actions` instead.
 
-            Use :py:attr:`additional_actions` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("additionalActions", "additional_actions", "3.0.0")
         return self.additional_actions
@@ -1288,17 +1292,17 @@ class Field(TreeObject):
 class Destination(TreeObject):
     """
     A class representing a destination within a PDF file.
+
     See section 8.2.1 of the PDF 1.6 reference.
 
     Args:
-      title: Title of this destination.
-      page: Reference to the page of this destination. Should
-        be an instance of :class:`IndirectObject<pypdf.generic.IndirectObject>`.
-      fit: How the destination is displayed.
+        title: Title of this destination.
+        page: Reference to the page of this destination. Should
+            be an instance of :class:`IndirectObject<pypdf.generic.IndirectObject>`.
+        fit: How the destination is displayed.
 
     Raises:
-      PdfReadError: If destination type is invalid.
-
+        PdfReadError: If destination type is invalid.
     """
 
     node: Optional[
@@ -1362,9 +1366,9 @@ class Destination(TreeObject):
 
     def getDestArray(self) -> "ArrayObject":  # deprecated
         """
-        .. deprecated:: 1.28.3
+        Use :py:attr:`dest_array` instead.
 
-            Use :py:attr:`dest_array` instead.
+        .. deprecated:: 1.28.3
         """
         deprecation_with_replacement("getDestArray", "dest_array", "3.0.0")
         return self.dest_array
@@ -1430,20 +1434,25 @@ class Destination(TreeObject):
 
     @property
     def color(self) -> Optional["ArrayObject"]:
-        """Read-only property accessing the color in (R, G, B) with values 0.0-1.0"""
+        """Read-only property accessing the color in (R, G, B) with values 0.0-1.0."""
         return self.get(
             "/C", ArrayObject([FloatObject(0), FloatObject(0), FloatObject(0)])
         )
 
     @property
     def font_format(self) -> Optional[OutlineFontFlag]:
-        """Read-only property accessing the font type. 1=italic, 2=bold, 3=both"""
+        """
+        Read-only property accessing the font type.
+
+        1=italic, 2=bold, 3=both
+        """
         return self.get("/F", 0)
 
     @property
     def outline_count(self) -> Optional[int]:
         """
         Read-only property accessing the outline count.
+
         positive = expanded
         negative = collapsed
         absolute value = number of visible descendents at all levels
